
#include <stdio.h>
#include <math.h>
 
#ifdef _WIN32
#include <windows.h>
/*
 * On some MS Windows compilers, the function "snprintf" is defined as
 * "_snprintf": 
 */
#define snprintf _snprintf
#else
/*
 * On MS Windows functions have to be exported with
 * __declspec(dllexport) so we define a dummy for other OS (like
 * Linux)
 */
#define __declspec(p)
#endif

// MicroSoft Visual C++ has problems with it's own min and max macros...
#ifdef min
#undef min
#endif
#ifdef max
#undef max
#endif


#include "mha_plugin.hh"
#include "mha_events.h"

/*
 * The definition of the signal processing run time configuration. The
 * constructor is used to initialize and validate the run time
 * configuration, based on the general (parser) configuration. If the
 * constructor failes, the parser command which caused the failure
 * will be rejected. The signal processing thread will not be affected.
 *
 * The constructor and destructor are called from the
 * control/configuration thread. All other functions and variables
 * should only be accessed from the signal processing thread. Read
 * only access from the configuration thread is possible if the read
 * operation is atomic.
 */
class cfg_t {
public:
    cfg_t(unsigned int,unsigned int);
    unsigned int channel;
};

/*
 * The definition of a simple signal processing class, containing a
 * constructor, a processing method and a parser variable.
 *
 * The base class MHAParser::parser_t provides the configuration
 * interface, the base class MHAPlugin::plugin_t provides standard
 * methods for \mha plugins (i.e. error handling) and functionality for
 * thread safe runtime configuration and algorithm communication.
 *
 * Implementation see below.
 */
class example6_t : public MHAPlugin::plugin_t<cfg_t> {
public:
    example6_t(algo_comm_t iac, const std::string & configured_name);
    mha_wave_t* process(mha_wave_t*);
    void prepare(mhaconfig_t&);
    float testtest[2];

private:
    void update_cfg();
    /* integer variable of MHA-parser: */
    MHAParser::int_t channel_no;
    MHAParser::int_t channel_no1;

    /* RMS level result variable (propagated to AC handle): */
    float variance_signal;
    comm_var_t test_ac;
    comm_var_t new_ac_var;
    comm_var_t modified_ac_var;
    std::vector<float> store_ac_single;
    //float store_ac_single[2][23];

    //float store_ac_single[2];
    //float store_ac_single[480];
    /** Handle to the ac space*/
    //const std::string & ac_name;


    

    
    MHAEvents::patchbay_t<example6_t> patchbay;
};

/*
 * This macro wraps the required ANSI-C interface around the algorithm
 * class. The first argument is the algorithm class name, the other
 * arguments define the input and output domain of the algorithm.
 */
MHAPLUGIN_CALLBACKS(steffencov,example6_t,wave,wave)

/*
 * Constructor of the runtime configuration class.
 *
 * This constructor is called in the configuration thread. This
 * allowes slow initialization processes (memory allocation,
 * configuration processing) even at run time.
 */

cfg_t::cfg_t(unsigned int ichannel,
             unsigned int numchannels)
    : channel(ichannel)
{
    if( channel >= numchannels )
        throw MHA_Error(__FILE__,__LINE__,
                        "Invalid channel number %u (only %u channels configured).",
                        channel,numchannels);
}

/*
 * Constructor of the simple signal processing class.
 */
example6_t::example6_t(algo_comm_t iac, const std::string &)
    : MHAPlugin::plugin_t<cfg_t>("Example rms level meter plugin",iac),
      /* initialzing variable 'channel_no' with MHAParser::int_t(char* name, .... ) */
      channel_no("channel in which the RMS level is measured","0","[0,["),
      channel_no1("Number of channels to calculate cov matrix for","0","[0,[")

{
    /* Register variables to the configuration parser: */
    insert_item("channel",&channel_no);
    insert_item("channel1",&channel_no1);

    /*
     * On write access to the parser variables a notify callback of
     * this class will be called. That funtion will update the runtime
     * configuration.
     */
    patchbay.connect(&channel_no.writeaccess,this,&example6_t::update_cfg);
    patchbay.connect(&channel_no1.writeaccess,this,&example6_t::update_cfg);

    /*
    test_ac.data_type = MHA_AC_FLOAT;
    test_ac.num_entries = 1;
    test_ac.stride = 2; //channel_no1.data;
    test_ac.data = store_ac_single;
    ac.insert_var( ac.handle, "test_ac2", test_ac);
    */
}

/*
 * Signal processing method. One channel of the input signal is scaled
 * by a factor. The factor and channel number can be changed at any
 * time using the configuration parser of the framework.
 */
mha_wave_t* example6_t::process(mha_wave_t* wave)
{
    poll_config();
    store_ac_single = MHA_AC::get_var_vfloat(ac,"SimpleStream");
    //std::cout << "value: \t" << store_ac_single[0] << "\t" << store_ac_single[1] << std::endl;
    //store_ac_single = store_ac_single+1;
    return wave;
}


void example6_t::prepare(mhaconfig_t& tfcfg)
{
        //ac.get_var(ac.handle, "example6_rmslev",&new_ac_var);


    if( tfcfg.domain != MHA_WAVEFORM )
        throw MHA_Error(__FILE__,__LINE__,"Example6: Only waveform processing is supported.");
    /* remember the transform configuration (i.e. channel numbers): */
    tftype = tfcfg;
    /* make sure that a valid runtime configuration exists: */
    update_cfg();
}

/*
 * This function adds a valid runtime configuration. In this case, the
 * channel number has to be configured for a valid configuration (see
 * prepare() callback).
 */
void example6_t::update_cfg()
{
    if( tftype.channels )
        push_config(new cfg_t(channel_no.data,tftype.channels));
}

MHAPLUGIN_DOCUMENTATION\
(steffencov,
 "example feature-extraction algorithm-communication",
 "This plugin calculates the RMS level of a given channel of the input signal,"
 " working in the time domain.\n"
 "The channel number is made accessible to the configuration structure and\n"
 "the result is stored into a algorithm communication variable (AC variable).")

// Local Variables:
// compile-command: "make"
// c-basic-offset: 4
// indent-tabs-mode: nil
// coding: utf-8-unix
// End:
